# Copyright 2022 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

import uuid

from odoo import api, fields, models


class PartnerRelease(models.Model):
    _name = "res.partner.release"
    _inherit = ["mail.thread", "mail.activity.mixin"]
    _description = "Suivi de mise en production logicielle"
    _order = "release_date desc"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    name = fields.Char("Nom", compute="_compute_name", copy=False)
    user_id = fields.Many2one(
        comodel_name="res.users",
        string="Utilisateur",
        required=True,
        ondelete="restrict",
        domain=[("share", "=", False)],
        tracking=1,
        default=lambda self: self.env.user,
    )
    partner_id = fields.Many2one(
        comodel_name="res.partner",
        string="Client",
        required=True,
        ondelete="restrict",
        domain=[("is_company", "=", True)],
        tracking=1,
    )
    release_date = fields.Datetime(
        string="Date de mise en production",
        default=lambda self: fields.Datetime.now(),
    )
    release_module_ids = fields.One2many(
        comodel_name="res.partner.release.module",
        inverse_name="release_id",
        string="Modules mis à jour",
        copy=False,
    )
    description = fields.Text("Description")
    access_token = fields.Char("Security Token", copy=False)
    release_share_link = fields.Char(
        string="Lien de partage client",
        compute="_compute_release_share_link",
    )

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends("partner_id", "release_date")
    def _compute_name(self):
        """
        Compute release name
        """
        for release in self:
            if release.release_date and release.partner_id:
                release.name = (
                    fields.Datetime.to_string(release.release_date)
                    + "-"
                    + release.partner_id.name
                )
            else:
                release.name = ""

    def _compute_release_share_link(self):
        for release in self:
            if release.partner_id and release.id and release.access_token:
                release.release_share_link = (
                    release.get_base_url()
                    + "/releases/"
                    + str(release.partner_id.id)
                    + "/"
                    + str(release.id)
                    + "?access_token="
                    + release.access_token
                )
            else:
                release.release_share_link = ""

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------
    @api.onchange("partner_id")
    def _onchange_partner_id(self):
        self.release_module_ids.unlink()

    # ------------------------------------------------------
    # Overrides parent
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------
    @api.model
    def create(self, vals):
        vals["access_token"] = str(uuid.uuid4())
        return super().create(vals)

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
